
#
# GENERATED WITH PDL::PP! Don't modify!
#
package PDL::Fit::Gaussian;

@EXPORT_OK  = qw( fitgauss1d fitgauss1dr );
%EXPORT_TAGS = (Func=>[@EXPORT_OK]);

use PDL::Core;
use PDL::Exporter;
use DynaLoader;



   
   @ISA    = ( 'PDL::Exporter','DynaLoader' );
   push @PDL::Core::PP, __PACKAGE__;
   bootstrap PDL::Fit::Gaussian ;




=head1 NAME

PDL::Fit::Gaussian - routines for fitting gaussians

=head1 DESCRIPTION


This module contains some custom gaussian fitting routines.
These were developed in collaboration with Alison Offer,
they do a reasonably robust job and are quite useful.

Gaussian fitting is something I do a lot of, so I figured
it was worth putting in my special code.

Note this code is also used in the Karma package.

=head1 SYNOPSIS

        use PDL::Fit::Gaussian;
        ($cen, $pk, $fwhm, $back, $err, $fit) = fitgauss1d($x, $data);
        ($pk, $fwhm, $back, $err, $fit) = fitgauss1dr($r, $data);

=head1 FUNCTIONS

=head2 fitgauss1d

=for ref

Fit 1D Gassian to data piddle

=for example

($cen, $pk, $fwhm2, $back, $err, $fit) = fitgauss1d($x, $data);

=for usage

($cen, $pk, $fwhm2, $back, $err, $fit) = fitgauss1d($x, $data);

=for signature

xval(n); data(n); [o]xcentre();[o]peak_ht(); [o]fwhm(); 
[o]background();int [o]err(); [o]datafit(n); 
[t]sig(n);  [t]xtmp(n); [t]ytmp(n); [t]yytmp(n); [t]rtmp(n);

Fit's a 1D Gaussian robustly free parameters are the centre, peak height,
FWHM. The background is NOT fit, because I find this is generally
unreliable, rather a median is determined in the 'outer' 10% of
pixels (i.e. those at the start/end of the data piddle). The initial
estimate of the FWHM is the length of the piddle/3, so it might fail
if the piddle is too long. (This is non-robust anyway). Most data
does just fine and this is a good default gaussian fitter.

SEE ALSO: fitgauss1dr() for fitting radial gaussians

=head2 fitgauss1dr

=for ref

Fit 1D Gassian to radial data piddle

=for example

($pk, $fwhm2, $back, $err, $fit) = fitgauss1dr($r, $data);

=for usage

($pk, $fwhm2, $back, $err, $fit) = fitgauss1dr($r, $data);

=for signature

xval(n); data(n); [o]peak_ht(); [o]fwhm(); 
[o]background();int [o]err(); [o]datafit(n); 
[t]sig(n);  [t]xtmp(n); [t]ytmp(n); [t]yytmp(n); [t]rtmp(n);

Fit's a 1D radial Gaussian robustly free parameters are the peak height,
FWHM. Centre is assumed to be X=0 (i.e. start of piddle).
The background is NOT fit, because I find this is generally
unreliable, rather a median is determined in the 'outer' 10% of
pixels (i.e. those at the end of the data piddle). The initial
estimate of the FWHM is the length of the piddle/3, so it might fail
if the piddle is too long. (This is non-robust anyway). Most data
does just fine and this is a good default gaussian fitter.

SEE ALSO: fitgauss1d() to fit centre as well.

=cut







sub PDL::fitgauss1d {
               my @a = @_;
               if($#a == 11 || $#a == -1 ) { &PDL::_fitgauss1d_int; }
                elsif($#a == 7) {
		 	$a[11] = PDL->nullcreate($a[0])
;$a[10] = PDL->nullcreate($a[0])
;$a[9] = PDL->nullcreate($a[0])
;$a[8] = PDL->nullcreate($a[0])
;
                       &PDL::_fitgauss1d_int(@a);@a=();
               } elsif($#a == 1) {
			my @ret;
			$a[11] = PDL->nullcreate($a[0]);
$a[10] = PDL->nullcreate($a[0]);
$a[9] = PDL->nullcreate($a[0]);
$a[8] = PDL->nullcreate($a[0]);
unshift @ret,($a[7] = PDL->nullcreate($a[0]) );
unshift @ret,($a[6] = PDL->nullcreate($a[0]) );
unshift @ret,($a[5] = PDL->nullcreate($a[0]) );
unshift @ret,($a[4] = PDL->nullcreate($a[0]) );
unshift @ret,($a[3] = PDL->nullcreate($a[0]) );
unshift @ret,($a[2] = PDL->nullcreate($a[0]) );

                       &PDL::_fitgauss1d_int(@a);@a=();
			return wantarray?(@ret):$ret[0];
		} else {
			barf "Invalid number of arguments for fitgauss1d";
		}
		}

*fitgauss1d = \&PDL::fitgauss1d;


sub PDL::fitgauss1dr {
               my @a = @_;
               if($#a == 10 || $#a == -1 ) { &PDL::_fitgauss1dr_int; }
                elsif($#a == 6) {
		 	$a[10] = PDL->nullcreate($a[0])
;$a[9] = PDL->nullcreate($a[0])
;$a[8] = PDL->nullcreate($a[0])
;$a[7] = PDL->nullcreate($a[0])
;
                       &PDL::_fitgauss1dr_int(@a);@a=();
               } elsif($#a == 1) {
			my @ret;
			$a[10] = PDL->nullcreate($a[0]);
$a[9] = PDL->nullcreate($a[0]);
$a[8] = PDL->nullcreate($a[0]);
$a[7] = PDL->nullcreate($a[0]);
unshift @ret,($a[6] = PDL->nullcreate($a[0]) );
unshift @ret,($a[5] = PDL->nullcreate($a[0]) );
unshift @ret,($a[4] = PDL->nullcreate($a[0]) );
unshift @ret,($a[3] = PDL->nullcreate($a[0]) );
unshift @ret,($a[2] = PDL->nullcreate($a[0]) );

                       &PDL::_fitgauss1dr_int(@a);@a=();
			return wantarray?(@ret):$ret[0];
		} else {
			barf "Invalid number of arguments for fitgauss1dr";
		}
		}

*fitgauss1dr = \&PDL::fitgauss1dr;



1; # OK




=head1 BUGS

May not converge for weird data, still pretty good!

=head1 AUTHOR

This file copyright (C) 1999, Karl Glazebrook (kgb@aaoepp.aao.gov.au),
Gaussian fitting code by Alison Offer
(aro@aaocbn.aao.gov.au).  All rights reserved. There
is no warranty. You are allowed to redistribute this software /
documentation under certain conditions. For details, see the file
COPYING in the PDL distribution. If this file is separated from the
PDL distribution, the copyright notice should be included in the file.


=cut



;



# Exit with OK status

1;

