
#
# GENERATED WITH PDL::PP! Don't modify!
#
package PDL::Slices;

@EXPORT_OK  = qw( affineinternal identity index index2d rld rle flowconvert converttypei clump xchg mv oneslice slice using affine diagonalI lags splitdim rotate threadI identvaff unthread dice dice_axis );
%EXPORT_TAGS = (Func=>[@EXPORT_OK]);

use PDL::Core;
use PDL::Exporter;
use DynaLoader;



   
   @ISA    = ( 'PDL::Exporter','DynaLoader' );
   push @PDL::Core::PP, __PACKAGE__;
   bootstrap PDL::Slices ;





=head1 NAME

PDL::Slices -- Stupid index tricks

=head1 SYNOPSIS

  use PDL;
  $a = ones(3,3);
  $b = $a->slice('-1:0,(1)');
  $c = $a->dummy(2);


=head1 DESCRIPTION

This package provides many of the powerful PerlDL core index
manipulation routines. These routines are usually two-way
so you can get a unit matrix by

	$a = zeroes(1000,1000);
	$a->diagonal(0,1) ++;

which is usually fairly efficient. See L<PDL::Indexing> and
L<PDL::Tips> for more examples.

These functions are usually two-way:

	$b = $a->slice("1:3");
	$b += 5;   		# $a is changed!

If you want to force a copy and no "flow" backwards, you need

	$b = $a->slice("1:3")->copy;
	$b += 5;		# $a is not changed.

alternatively, you can use

	$b = $a->slice("1:3")->sever;

which doesn't copy the struct but beware that after

	$b = $a->slice("1:3");
	$c = $b->sever;

the variables $b and $c point to the same object but with ->copy
they do not.

The fact that there is this kind of flow makes PDL a very
powerful language in many ways: since you can alter the original
data by altering some easier-to-use representation of it, many things are
much easier to accomplish, just like making the above unit matrix.

=cut

use PDL::Core ':Internal';







=head1 FUNCTIONS



=cut






=head2 affineinternal

=for sig

  Signature: (P(); C())

=for ref

internal

=cut







	*affineinternal = \&PDL::affineinternal;
	sub PDL::affineinternal {
		my $this = shift;
		my $foo=$this->null;
		PDL::affineinternal_XX($this,$foo,@_);
		$foo
	}
	




=head2 identity

=for sig

  Signature: (P(); C())

=for ref

internal

=cut







	*identity = \&PDL::identity;
	sub PDL::identity {
		my $this = shift;
		my $foo=$this->null;
		PDL::identity_XX($this,$foo,@_);
		$foo
	}
	




=head2 index

=for sig

  Signature: (a(n); int ind(); [oca] c())

=for ref

These functions provide rudimentary index indirection.

=for example

	c = a(ind());
	c = a(ind1(),ind2());

It would be useful to have a more complete function for this
at some point, or at least a perl wrapper, that allows

	$c = $a->islice("1:2",$ind1,"3:4",$ind2");

with many dimensions.

This function is two-way, i.e. after

        $c = $a->index(pdl[0,5,8]);
        $c .= pdl [0,2,4];

the changes in $c will flow back to $a.



=cut




sub PDL::index {
               my @a = @_;
               if($#a == 2 || $#a == -1 ) { &PDL::_index_int; }
                elsif($#a == 2) {
		 	
                       &PDL::_index_int(@a);@a=();
               } elsif($#a == 1) {
			my @ret;
			unshift @ret,($a[2] = PDL->nullcreate($a[0]) );

                       &PDL::_index_int(@a);@a=();
			return wantarray?(@ret):$ret[0];
		} else {
			barf "Invalid number of arguments for index";
		}
		}

*index = \&PDL::index;




=head2 index2d

=for sig

  Signature: (a(na,nb); int inda(); int indb(); [oca] c())

=for ref

These functions provide rudimentary index indirection.

=for example

	c = a(ind());
	c = a(ind1(),ind2());

It would be useful to have a more complete function for this
at some point, or at least a perl wrapper, that allows

	$c = $a->islice("1:2",$ind1,"3:4",$ind2");

with many dimensions.

This function is two-way, i.e. after

        $c = $a->index(pdl[0,5,8]);
        $c .= pdl [0,2,4];

the changes in $c will flow back to $a.



=cut




sub PDL::index2d {
               my @a = @_;
               if($#a == 3 || $#a == -1 ) { &PDL::_index2d_int; }
                elsif($#a == 3) {
		 	
                       &PDL::_index2d_int(@a);@a=();
               } elsif($#a == 2) {
			my @ret;
			unshift @ret,($a[3] = PDL->nullcreate($a[0]) );

                       &PDL::_index2d_int(@a);@a=();
			return wantarray?(@ret):$ret[0];
		} else {
			barf "Invalid number of arguments for index2d";
		}
		}

*index2d = \&PDL::index2d;




=head2 rld

=for sig

  Signature: (int a(n); b(n); [o]c(m))

=for ref

Run-length decode a vector

Given a vector $a of the numbers of instances of values $b, run-length
decode to $c.

=for example

rld($a,$b,$c=null);



=cut




sub PDL::rld {
  my ($a,$b) = @_;
  my ($c);
  if ($#_ == 2) {
    $c = $_[2];
  } else {
# XXX Need to improve emulation of threading in auto-generating c
    my ($size) = $a->sumover->max;
    my (@dims) = $a->dims;
    shift @dims;
    $c = $b->zeroes($size,@dims);
  }
  &PDL::_rld_int($a,$b,$c);
  $c;
}


*rld = \&PDL::rld;




=head2 rle

=for sig

  Signature: (c(n); int [o]a(n); [o]b(n))

=for ref

Run-length encode a vector

Given vector $c, generate a vector $a with the number of each element,
and a vector $b of the unique values.  Only the elements up to the
first instance of `0' in $a should be considered.

=for example

rle($c,$a=null,$b=null);



=cut




sub PDL::rle {
               my @a = @_;
               if($#a == 2 || $#a == -1 ) { &PDL::_rle_int; }
                elsif($#a == 2) {
		 	
                       &PDL::_rle_int(@a);@a=();
               } elsif($#a == 0) {
			my @ret;
			unshift @ret,($a[2] = PDL->nullcreate($a[0]) );
unshift @ret,($a[1] = PDL->nullcreate($a[0]) );

                       &PDL::_rle_int(@a);@a=();
			return wantarray?(@ret):$ret[0];
		} else {
			barf "Invalid number of arguments for rle";
		}
		}

*rle = \&PDL::rle;




=head2 flowconvert

=for sig

  Signature: (PARENT(); [oca]CHILD(); int totype)

=for ref

internal

=cut




sub PDL::flowconvert {
               my @a = @_;
               if($#a == 2 || $#a == -1 ) { &PDL::_flowconvert_int; }
                elsif($#a == 2) {
		 	
                       &PDL::_flowconvert_int(@a);@a=();
               } elsif($#a == 1) {
			my @ret;
			$a[2] = $a[1];
unshift @ret,($a[1] = PDL->nullcreate($a[0]) );

                       &PDL::_flowconvert_int(@a);@a=();
			return wantarray?(@ret):$ret[0];
		} else {
			barf "Invalid number of arguments for flowconvert";
		}
		}

*flowconvert = \&PDL::flowconvert;




=head2 converttypei

=for sig

  Signature: (P(); C(); int totype)

=for ref

internal

=cut







	*converttypei = \&PDL::converttypei;
	sub PDL::converttypei {
		my $this = shift;
		my $foo=$this->null;
		PDL::converttypei_XX($this,$foo,@_);
		$foo
	}
	




=head2 clump

=for sig

  Signature: (P(); C(); int n)


=for ref

"clumps" the first n dimensions into one large dimension

If, for example,  $a has dimensions (5,3,4) then after

=for example

	$b = $a->clump(2);   # Clump 2 first dimensions

the variable $b will have dimensions (15,4)
and the element $b->at(7,3) refers to the element $a->at(1,2,3).



=cut







	*clump = \&PDL::clump;
	sub PDL::clump {
		my $this = shift;
		my $foo=$this->null;
		PDL::clump_XX($this,$foo,@_);
		$foo
	}
	




=head2 xchg

=for sig

  Signature: (P(); C(); int n1; int n2)

=for ref

exchange two dimensions

The command

=for example

	$b = $a->xchg(2,3);

creates $b to be like $a except that the dimensions 2 and 3
are exchanged with each other i.e.

	$b->at(5,3,2,8) == $a->at(5,3,8,2)



=cut







	*xchg = \&PDL::xchg;
	sub PDL::xchg {
		my $this = shift;
		my $foo=$this->null;
		PDL::xchg_XX($this,$foo,@_);
		$foo
	}
	




=head2 mv

=for sig

  Signature: (P(); C(); int n1; int n2)

=for ref

move a dimension to another position

The command

=for example

	$b = $a->mv(4,1);

creates $b to be like $a except that the dimension 4 is moved to the
place 1:

	$b->at(1,2,3,4,5,6) == $a->at(1,5,2,3,4,6);

The other dimensions are moved accordingly.



=cut







	*mv = \&PDL::mv;
	sub PDL::mv {
		my $this = shift;
		my $foo=$this->null;
		PDL::mv_XX($this,$foo,@_);
		$foo
	}
	




=head2 oneslice

=for sig

  Signature: (P(); C(); int nth; int from; int step; int nsteps)

=for ref

experimental function - not for public use

=for example

  $a = oneslice();

This is not for public use currently. See the source if you have to.
This function can be used to accomplish run-time changing of
transformations i.e. changing the size of some piddle at run-time.

However, the mechanism is not yet finalized and this is just a demonstration.



=cut







	*oneslice = \&PDL::oneslice;
	sub PDL::oneslice {
		my $this = shift;
		my $foo=$this->null;
		PDL::oneslice_XX($this,$foo,@_);
		$foo
	}
	




=head2 slice

=for sig

  Signature: (P(); C(); char* str)

=for ref

Returns a rectangular slice of the original piddle

=for example

  $a->slice('1:3');  #  return the second to fourth elements of $a
  $a->slice('3:1');  #  reverse the above
  $a->slice('-2:1'); #  return last-but-one to second elements of $a

The argument string is a comma-separated list of what to do
for each dimension. The current formats include
the following, where I<a>, I<b> and I<c> are integers and can
take legal array index values (including -1 etc):

=over 8

=item ":"

takes the whole dimension intact.

=item ""

(nothing) is a synonym for ":"
(This means that C<$a-E<gt>slice(':,3')> is equal to $a-E<gt>slice(',3')).

=item "a"

slices only this value out of the corresponding dimension.

=item "(a)"

means the same as "a" by itself except that the resulting
dimension of length one is deleted (so if $a has dims (3,4,5) then
C<$a-E<gt>slice(':,(2),:')> has dimensions (3,5) whereas
$a-E<gt>slice(':,2,:') has dimensions (3,1,5)).

=item "a:b"

slices the range I<a> to I<b> inclusive out of the dimension.

=item "a:b:c"

slices the range I<a> to I<b>, with step I<c> (i.e. 3:7:2 gives the indices
(3,5,7)). This may be confusing to Matlab users but several other
packages already use this syntax.

=item "*"

inserts an extra dimension of width 1 and

=item "*a"

inserts an extra (dummy) dimension of width I<a>.

=back

An extension is planned for a later stage allowing
$a->slice('(=1),(=1|5:8),3:6(=1),4:6') to express a multidimensional
diagonal of $a.



=cut







	*slice = \&PDL::slice;
	sub PDL::slice {
		my $this = shift;
		my $foo=$this->null;
		PDL::slice_XX($this,$foo,@_);
		$foo
	}
	



=head2 using

=for ref

Returns array of column numbers requested

=for usage

 line $pdl->using(1,2);

 Plot line of column 1 of $pdl vs. column 2

=for example

 perldl> $pdl = rcols("file");

 perldl> line $pdl->using(1,2);

=cut

*using = \&PDL::using;
sub PDL::using {
  my ($x,@ind)=@_;
  @ind = list $ind[0] if (ref $ind[0] eq 'PDL');
  foreach (@ind) {
    $_ = $x->slice("($_)");
  }
  @ind;
}





=head2 affine

=for sig

  Signature: (P(); C(); int offspar; SV *dimlist; SV *inclist)

=for ref

internal

=cut







	*affine = \&PDL::affine;
	sub PDL::affine {
		my $this = shift;
		my $foo=$this->null;
		PDL::affine_XX($this,$foo,@_);
		$foo
	}
	




=head2 diagonalI

=for sig

  Signature: (P(); C(); SV *list)

=for ref

Returns the multidimensional diagonal over the specified dimensions.

The diagonal is placed at the first (by number) dimension that is
diagonalized.
The other diagonalized dimensions are removed. So if $a has dimensions
(5,3,5,4,6,5) then after

=for example

	$b = $a->diagonal(0,2,5);

the piddle $b has dimensions (5,3,4,6) and $b->at(2,1,0,1) refers
to $a->at(2,1,2,0,1,2).

NOTE: diagonal doesn't handle threadids correctly. XXX FIX



=cut







	*diagonalI = \&PDL::diagonalI;
	sub PDL::diagonalI {
		my $this = shift;
		my $foo=$this->null;
		PDL::diagonalI_XX($this,$foo,@_);
		$foo
	}
	




=head2 lags

=for sig

  Signature: (P(); C(); int nthdim; int step; int n)

=for ref

Returns a piddle of lags to parent.

I.e. if $a contains

       [0,1,2,3,4,5,6,7]

then

=for example

	$b = $a->lags(0,2,2);

 is a (5,2) matrix

       [2,3,4,5,6,7]
       [0,1,2,3,4,5]

This order of returned indices is kept because the function is
called "lags" i.e. the nth lag is n steps behind the original.



=cut







	*lags = \&PDL::lags;
	sub PDL::lags {
		my $this = shift;
		my $foo=$this->null;
		PDL::lags_XX($this,$foo,@_);
		$foo
	}
	




=head2 splitdim

=for sig

  Signature: (P(); C(); int nthdim; int nsp)

=for ref

Splits a dimension in the parent piddle (opposite of C<clump>)

After

=for example

	$b = $a->splitdim(2,3);

the expression

	$b->at(6,4,x,y,3,6) == $a->at(6,4,x+3*y)

is always true (x has to be less than 3).



=cut







	*splitdim = \&PDL::splitdim;
	sub PDL::splitdim {
		my $this = shift;
		my $foo=$this->null;
		PDL::splitdim_XX($this,$foo,@_);
		$foo
	}
	




=head2 rotate

=for sig

  Signature: (x(n); int shift(); [oca]y(n))

=for ref

Shift vector elements along with wrap. Flows data back&forth.



=cut




sub PDL::rotate {
               my @a = @_;
               if($#a == 2 || $#a == -1 ) { &PDL::_rotate_int; }
                elsif($#a == 2) {
		 	
                       &PDL::_rotate_int(@a);@a=();
               } elsif($#a == 1) {
			my @ret;
			unshift @ret,($a[2] = PDL->nullcreate($a[0]) );

                       &PDL::_rotate_int(@a);@a=();
			return wantarray?(@ret):$ret[0];
		} else {
			barf "Invalid number of arguments for rotate";
		}
		}

*rotate = \&PDL::rotate;




=head2 threadI

=for sig

  Signature: (P(); C(); int id; SV *list)

=for ref

internal

Put some dimensions to a threadid.

=for example

	$b = $a->threadI(0,1,5); # thread over dims 1,5 in id 1



=cut







	*threadI = \&PDL::threadI;
	sub PDL::threadI {
		my $this = shift;
		my $foo=$this->null;
		PDL::threadI_XX($this,$foo,@_);
		$foo
	}
	




=head2 identvaff

=for sig

  Signature: (P(); C())

=for ref

A vaffine identity transformation (includes thread_id copying).

Mainly for internal use.



=cut







	*identvaff = \&PDL::identvaff;
	sub PDL::identvaff {
		my $this = shift;
		my $foo=$this->null;
		PDL::identvaff_XX($this,$foo,@_);
		$foo
	}
	




=head2 unthread

=for sig

  Signature: (P(); C(); int atind)

=for ref

All threaded dimensions are made real again.

See [TBD Doc] for details and examples.



=cut







	*unthread = \&PDL::unthread;
	sub PDL::unthread {
		my $this = shift;
		my $foo=$this->null;
		PDL::unthread_XX($this,$foo,@_);
		$foo
	}
	


;


=head2 reorder

=for ref

Re-orders the dimensions of a PDL based on the supplied list.

Similar to the L<xchg> method, this method re-orders the dimensions
of a PDL. While the L<xchg> method swaps the position of two dimensions,
the reorder method can change the positions of many dimensions at
once.

=for usage


 $reOrderedPDL = $pdl->reorder(5,4,3,2,1,0); # Completely reverse the 
						 # dimension order of a 6-Dim
						 # array.

The argument to reorder is an array representing where the current dimensions
should go in the new array. In the above usage, the argument to reorder (5,4,3,2,1,0)
indicates that the old dimensions ($pdl's dims) should be re-arranged to make the
new pdl ($reOrderPDL) according to the following:

   Old Position   New Position
   ------------   ------------
   5              0
   4              1
   3 		  2
   2		  3
   1		  4
   0		  5


=for example 

Example:

 perldl> $a = sequence(5,3,2);	  # Create a 3-d Array
 perldl> p $a

 [
  [
   [ 0  1  2  3  4]
   [ 5  6  7  8  9]
   [10 11 12 13 14]
  ]
  [
   [15 16 17 18 19]
   [20 21 22 23 24]
   [25 26 27 28 29]
  ]
 ]

 perldl> p $a->reorder(2,1,0); # Reverse the order of the 3-D PDL

 [
  [
   [ 0 15]
   [ 5 20]
   [10 25]
  ]
  [
   [ 1 16]
   [ 6 21]
   [11 26]
  ]
  [
   [ 2 17]
   [ 7 22]
   [12 27]
  ]
  [
   [ 3 18]
   [ 8 23]
   [13 28]
  ]
  [
   [ 4 19]
   [ 9 24]
   [14 29]
  ]
 ]

The above is a simple example that could be duplicated by calling
$a->xchg(0,2), but it demonstrates the basic functionality of reorder.

As this is an index function, any modifications to the
result PDL will change the parent.

=cut

sub PDL::reorder {
	my ($pdl,@newDimOrder) = @_;
	
	my $arrayMax = $#newDimOrder;
  
	#Error Checking:
	if( $pdl->getndims != scalar(@newDimOrder) ){
		my $errString = "PDL::reoderDims: Number of elements (".scalar(@newDimOrder).") in newDimOrder array doesn't\n";
		$errString .= "match the number of dims in the supplied PDL ($ndims)\n";
		barf($errString);
	}
	# a quicker way to do the reorder
	return $pdl->thread(@newDimOrder)->unthread(0);
}




=head2 dice

=for ref

Dice rows/columns/planes out of a PDL using indexes for
each dimension.

This function can be used to extract irregular subsets
along many dimension of a PDL, e.g. only certain rows in an image,
or planes in a cube. This can of course be done with
the usual dimension tricks but this saves having to
figure it out each time!

This method is similar in functionality to the L<slice>
method, but L<slice> requires that contiguous ranges or ranges
with constant offset be extracted. ( i.e. L<slice> requires 
ranges of the form 1,2,3,4,5 or 2,4,6,8,10). Because of this
restriction, L<slice> is more memory efficient and slightly faster
than dice

=for usage

 $slice = $data->dice([0,2,6],[2,1,6]); # Dicing a 2-D array

The arguments to dice are arrays (or 1D PDLs) for each dimension
in the PDL. These arrays are used as indexes to which rows/columns/cubes,etc
to dice-out (or extract) from the $data PDL. 

=for example 

 perldl> $a = sequence(10,4)
 perldl> p $a

 [
  [ 0  1  2  3  4  5  6  7  8  9]
  [10 11 12 13 14 15 16 17 18 19]
  [20 21 22 23 24 25 26 27 28 29]
  [30 31 32 33 34 35 36 37 38 39]
 ]

 perldl> p $a->dice([1,2],[0,3]) # Select columns 1,2 and rows 0,3

 [
  [ 1  2]
  [31 32]
 ]

As this is an index function, any modifications to the
slice change the parent.

=cut

sub PDL::dice { 

	my $self = shift;
	my @dim_indexes = @_;  # array of dimension indexes
	
	#Check that the number of dim indexes = number of dimensions in the PDL
	my $no_indexes = scalar(@dim_indexes);
	my $noDims = $self->getndims;
	barf("PDL::dice: Number of index arrays ($no_indexes) not equal to the dimensions of the PDL ($noDims")
			 if $no_indexes != $noDims;
	my $index;
	my $pdlIndex;
	my $outputPDL=$self;
	my $indexNo = 0;

	# Go thru each index array and dice the input PDL:
	foreach $index(@dim_indexes){

		$outputPDL = $outputPDL->dice_axis($indexNo,$index);

		$indexNo++;
	}

	return $outputPDL;
}  
*dice = \&PDL::dice;


=head2 dice_axis

=for ref

Dice rows/columns/planes from a single PDL axis (dimension)
using index along a specified axis

This function can be used to extract irregular subsets
along any dimension, e.g. only certain rows in an image,
or planes in a cube. This can of course be done with
the usual dimension tricks but this saves having to
figure it out each time!

=for usage

 $slice = $data->dice_axis($axis,$index);

=for example

 perldl> $a = sequence(10,4)
 perldl> $idx = pdl(1,2)
 perldl> p $a->dice_axis(0,$idx) # Select columns

 [
  [ 1  2]
  [11 12]
  [21 22]
  [31 32]
 ]

 perldl> $t = $a->dice_axis(1,$idx) # Select rows
 perldl> $t.=0
 perldl> p $a

 [
  [ 0  1  2  3  4  5  6  7  8  9]
  [ 0  0  0  0  0  0  0  0  0  0]
  [ 0  0  0  0  0  0  0  0  0  0]
  [30 31 32 33 34 35 36 37 38 39]
 ]

The trick to using this is that the index selects
elements along the dimensions specified, so if you
have a 2D image axis=0 will select certain X values
- i.e. extract columns

As this is an index function, any modifications to the
slice change the parent.

=cut

sub PDL::dice_axis { 
  my($self,$axis,$idx) = @_;
  
  # Convert to PDLs: array refs using new, otherwise use topdl:
  my $ix = (ref($idx) eq 'ARRAY') ? ref($self)->new($idx) : ref($self)->topdl($idx);
  my $n = $self->getndims;
  my $a = $ix->getndims;
  barf("index_axis: index must be <=1D") if $a>1;
  for ($a..$n-1) {
     $ix = $ix->dummy(0);
  }
  return $self->mv($axis,0)->index($ix)->mv($n-1,$axis);
}  
*dice_axis = \&PDL::dice_axis;

=head1 AUTHOR

Copyright (C) 1997 Tuomas J. Lukka.
All rights reserved. There is no warranty. You are allowed
to redistribute this software / documentation under certain
conditions. For details, see the file COPYING in the PDL
distribution. If this file is separated from the PDL distribution,
the copyright notice should be included in the file.


=cut





# Exit with OK status

1;

